/* -*- c -*-
 * Time-stamp: <2005-04-16 19:51:13 rsmith>
 * 
 * lamprop/calc.c
 * Copyright (C) 2002--2005 R.F. Smith <rsmith@xs4all.nl>. 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* If you don't want assertions, #define NDEBUG */
/*#define NDEBUG*/

#include <assert.h>
#include <stdio.h>
#include <math.h>

#include "macros.h"
#include "lamprop.h"
#include "matrix.h"

/* This enables you to see which files have been used in building a
 * binary. N.B, __attribute is a GCC extension, so we test for __GNUC__. */
#ifdef __GNUC__
static char *SCMID __attribute__ ((unused)) = 
"$Id: calc.c 33 2005-04-16 21:47:48Z rsmith $";
#endif

#define EPSILON 1e-7

/* first pass; calculate the elastic properties of the k-th lamina */
void lamina_properties(struct laminate *pl, int k)
{
	double E1f, E2f, G12f, v12f, Em, Gm, vm, vf;
	double cte1f, cte2f, ctem, cte1, cte2;
	double S11, S12, S22, S66;
	double Q11, Q12, Q22, Q66;
	double denum, foo, bar;
	double n, n2, n3, n4, m, m2, m3, m4;
	struct lamina *pla;
	struct matrix *pm;
	assert(pl != NULL);
	pla = (struct lamina *)list_nth(LISTP(pl->lamina), k);
	if (pla == NULL) {
		fatal("\"%s\": invalid lamina index %d; number of lamina %d\n",
		      pl->name, k, pl->nl);
	}
	pm = pl->matrix;
	if (pm == NULL) {
		fatal("no matrix defined in laminate \"%s\"\n", pl->name);
	}
	vf = pl->vf;
	E1f = pla->f->E1; E2f = pla->f->E2;
	G12f = pla->f->G12; v12f = pla->f->v12;
	Em = pm->E; Gm = pm->G; vm = pm->v;
	pla->E1 = E1f*vf+Em*(1-vf);
	pla->E2 = 1/((vf/E2f+0.5*(1-vf)/Em)/(vf+0.5*(1-vf)));
	pla->G12 = 1/((vf/G12f+0.6*(1-vf)/Gm)/(vf+0.6*(1-vf)));
	pla->v12 = v12f*vf+vm*(1-vf);
	cte1f = pla->f->cte1; cte2f = pla->f->cte2;
	ctem = pm->cte;
	cte1 = (cte1f*E1f*vf+ctem*Em*(1-vf))/pla->E1;	/* [1], p. 142 */
	cte2 = ctem+(cte2f-ctem)*vf+			/* [1], p. 143 */
		((E1f*vm-Em*v12f)/pla->E1)*(ctem-cte1f)*(1-vf)*vf;
	pla->m = m = cos(pla->angle);
	pla->n = n = sin(pla->angle);
	n2 = n*n; n3 = n2*n; n4 = n3*n;
	m2 = m*m; m3 = m2*m; m4 = m3*m;
	pla->cte_x = cte1*m2+cte2*n2;	/* [1], p. 199 */
	pla->cte_y = cte1*n2+cte2*m2;
	pla->cte_xy = 2*(cte1-cte2)*m*n;
	S11 = 1/pla->E1; S12 = -pla->v12/pla->E1;
	S22 = 1/pla->E2; S66 = 1/pla->G12;
	denum = S11*S22-S12*S12;
	Q11 = S22/denum; Q12 = -S12/denum; Q22 = S11/denum; Q66 = 1/S66;
	pla->Q_11 = Q11*m4+2*(Q12+2*Q66)*n2*m2+Q22*n4;
	foo = Q11-Q12-2*Q66; bar = Q12-Q22+2*Q66;
	pla->Q_12 = (Q11+Q22-4*Q66)*n2*m2+Q12*(n4+m4);
	pla->Q_16 = foo*n*m3+bar*n3*m;
	pla->Q_22 = Q11*n4+2*(Q12+2*Q66)*n2*m2+Q22*m4;
	pla->Q_26 = foo*n3*m+bar*n*m3;
	pla->Q_66 = (Q11+Q22-2*Q12-2*Q66)*n2*m2+Q66*(n4+m4);
	pla->t = pla->weight/(pla->f->rho*1000)*(1+(1-vf)/vf);
	pla->rho = pla->f->rho*vf+pl->matrix->rho*(1-vf);
	pl->t += pla->t;
	debug("lamina %d: Q_11=%g Q_12=%g Q_16=%g", k+1, pla->Q_11, 
	      pla->Q_12, pla->Q_16);
	debug("Q_22=%g Q_26=%g Q_66=%g t=%g", pla->Q_22, pla->Q_26, 
	      pla->Q_66, pla->t);	
}

/* second pass; calculate the geometric properties of a lamina */
void lamina_geometry(struct laminate *pl, int k)
{
	double zs, ze;
	struct lamina *pla, *ppla;
	assert(pl != NULL);
	pla = (struct lamina *)list_nth(LISTP(pl->lamina), k);
	if (pla == NULL) {
		fatal("\"%s\": invalid lamina index %d; number of lamina %d\n",
		      pl->name, k, pl->nl);
	}
	if (k == 0) {
		pla->z0 = -pl->t/2;
	} else {
		ppla = (struct lamina *)list_prev(LISTP(pla));
		pla->z0 = ppla->z0 + ppla->t;
	}
	zs = pla->z0;
	ze = zs + pla->t;
	debug("lam. %d; zs=%g, ze=%g", k+1, zs, ze);
	pla->z2 = (ze*ze-zs*zs)/2;
	pla->z3 = (ze*ze*ze-zs*zs*zs)/3;
}

void ABD(struct laminate *pl)
{
	struct lamina *pla;
	int i,j;
	assert(pl != NULL);
	for (i=0; i<6; i++) {
		for (j=0; j<6; j++) {
			pl->ABD[i][j] = 0;
		}
	}
	pl->Nt_x = pl->Nt_y = pl->Nt_xy = 0;
	pl->rho = 0;
	pla = (struct lamina *)list_head(LISTP(pl->lamina));
	while (pla) {
		/* first row */
		pl->ABD[0][0] += pla->Q_11*pla->t;	/* [1], p. 290 */
		pl->ABD[0][1] += pla->Q_12*pla->t;
		pl->ABD[0][2] += pla->Q_16*pla->t;
		pl->ABD[0][3] += pla->Q_11*pla->z2;
		pl->ABD[0][4] += pla->Q_12*pla->z2;
		pl->ABD[0][5] += pla->Q_16*pla->z2;
		/* second row */
		pl->ABD[1][0] += pla->Q_12*pla->t;
		pl->ABD[1][1] += pla->Q_22*pla->t;
		pl->ABD[1][2] += pla->Q_26*pla->t;
		pl->ABD[1][3] += pla->Q_12*pla->z2;
		pl->ABD[1][4] += pla->Q_22*pla->z2;
		pl->ABD[1][5] += pla->Q_26*pla->z2;
		/* third row */
		pl->ABD[2][0] += pla->Q_16*pla->t;
		pl->ABD[2][1] += pla->Q_26*pla->t;
		pl->ABD[2][2] += pla->Q_66*pla->t;
		pl->ABD[2][3] += pla->Q_16*pla->z2;
		pl->ABD[2][4] += pla->Q_26*pla->z2;
		pl->ABD[2][5] += pla->Q_66*pla->z2;
		/* fourth row */
		pl->ABD[3][0] += pla->Q_11*pla->z2;
		pl->ABD[3][1] += pla->Q_12*pla->z2;
		pl->ABD[3][2] += pla->Q_16*pla->z2;
		pl->ABD[3][3] += pla->Q_11*pla->z3;
		pl->ABD[3][4] += pla->Q_12*pla->z3;
		pl->ABD[3][5] += pla->Q_16*pla->z3;
		/* fifth row */
		pl->ABD[4][0] += pla->Q_12*pla->z2;
		pl->ABD[4][1] += pla->Q_22*pla->z2;
		pl->ABD[4][2] += pla->Q_26*pla->z2;
		pl->ABD[4][3] += pla->Q_12*pla->z3;
		pl->ABD[4][4] += pla->Q_22*pla->z3;
		pl->ABD[4][5] += pla->Q_26*pla->z3;
		/* sixth row */
		pl->ABD[5][0] += pla->Q_16*pla->z2;
		pl->ABD[5][1] += pla->Q_26*pla->z2;
		pl->ABD[5][2] += pla->Q_66*pla->z2;
		pl->ABD[5][3] += pla->Q_16*pla->z3;
		pl->ABD[5][4] += pla->Q_26*pla->z3;
		pl->ABD[5][5] += pla->Q_66*pla->z3;
		/* Calculate unit thermal stress resultants */
		pl->Nt_x += (pla->Q_11*pla->cte_x+pla->Q_12*pla->cte_y+
			pla->Q_16*pla->cte_xy)*pla->t;	/* [1], p. 445 */
		pl->Nt_y += (pla->Q_12*pla->cte_x+pla->Q_22*pla->cte_y+
			pla->Q_26*pla->cte_xy)*pla->t;
		pl->Nt_xy += (pla->Q_16*pla->cte_x+pla->Q_26*pla->cte_y+
			pla->Q_66*pla->cte_xy)*pla->t;
		/* denisty calculation */
		pl->rho += pla->rho*pla->t;
		/* next lamina */
		pla = (struct lamina *)list_next(LISTP(pla));
	}
	/* Set small values in ABD to 0 */
	for (i=0; i<6; i++){
		for (j=0; j<6; j++){
			if (fabs(pl->ABD[i][j]) < EPSILON) {
				pl->ABD[i][j] = 0.0;
			}
		}
	}
	/* Calculate the abd matrix, inverse of ABD */
	if(mat_inv(6, pl->ABD, pl->abd) != 0) {
		warning("inverting the ABD matrix failed\n");
	}
	/* Calculate the engineering properties */
	pl->Ex = (pl->ABD[0][0]*pl->ABD[1][1]-pl->ABD[0][1]*pl->ABD[0][1])/
		(pl->ABD[1][1]*pl->t);	/* [1], p. 326 */
	pl->Ey = (pl->ABD[0][0]*pl->ABD[1][1]-pl->ABD[0][1]*pl->ABD[0][1])/
		(pl->ABD[0][0]*pl->t);
	pl->Gxy = pl->ABD[2][2]/pl->t;
	pl->Vxy = pl->ABD[0][1]/pl->ABD[1][1];
	pl->Vyx = pl->ABD[0][1]/pl->ABD[0][0];
	/* Calculate the coefficients of thermal expansion */
	pl->cte_x = pl->abd[0][0]*pl->Nt_x + pl->abd[0][1]*pl->Nt_y + 
		pl->abd[0][2]*pl->Nt_xy;
	pl->cte_y = pl->abd[1][0]*pl->Nt_x + pl->abd[1][1]*pl->Nt_y + 
		pl->abd[1][2]*pl->Nt_xy;
	/* Finish density calculation */
	pl->rho /= pl->t;
}

/* EOF calc.c */
