/* -*- c -*-
 * Time-stamp: <2005-04-16 20:50:00 rsmith>
 * 
 * lamprop/lamprop.h
 * Copyright (C) 2002--2005 R.F. Smith <rsmith@xs4all.nl>. 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef LAMPROP_H
#define LAMPROP_H

#ifdef __cplusplus
#  define BEGIN_C_DECLS extern "C" {
#  define END_C_DECLS   }
#else                           /* !__cplusplus */
#  define BEGIN_C_DECLS
#  define END_C_DECLS
#endif                          /* __cplusplus */

#ifndef M_PI            /* For Mingw32 */
#define M_PI 3.14159265358979323846
#endif

#include "list.h"	/* now a separate module */

BEGIN_C_DECLS

enum {
	OF_ENGPROP = (1<<0),	/* print only engineering properties */
	OF_MAT = (1<<1),	/* print only the ABD and abd matrices */
/*	OF_FILE = (1<<2),*/	/* output to file, instead of stdout */
};

/* Properties of a fiber. */
struct fiber 
{
	struct list l;	/* list node, _must_ be the first member */
	char *name;
	/* fiber properties */
	double E1;	/* modulus in fiber direction [MPa] (N/mm^2) */
	double E2;	/* modulus perp. to fiber direction [MPa] */
	double v12;	/* poisson constant [-] */
	double G12;	/* shear modulus of fiber [MPa] */
	double cte1;	/* long. coefficient of thermal exp. [K^-1] */
	double cte2;	/* transv. coefficient of thermal exp. [K^-1] */
	/* double ten_st1; */	/* tensile strength [MPa] */
	/* double comp_st1; *//* compressive strength [MPa] */
	/* double sh_st12; */	/* shear strength [MPa] */
	double rho;	/* fiber density [gr/cm3] */
};

/* Properties of a matrix material. */
struct matrix 
{
	struct list l;	/* list node, _must_ be the first member */
	char *name;
	/* matrix properties */
	double E;	/* Young's modulus [MPa] */
	double v;	/* poisson constant [-] */
	double cte;	/* coefficient of thermal expansion [K^-1] */
	/* double ten_st; */	/* tensile strength [MPa] */
	double rho;	/* matrix density [gr/cm3] */
	/* calculated properties */
	double G;	/* shear modulus of matrix [MPa] [G=E/(2(1+v))] */
	double sh_st;	/* shear strength [MPa] */
};

/* Properties of a single UD layer. */
struct lamina 
{
	struct list l;		/* list node, _must_ be the first member */
	struct fiber *f;	/* the fiber that maks up this layer */
	double weight;		/* [gr/m2] */
	double angle;		/* from x-axis, in degrees */
	/* calculated properties of a layer, first pass */
	double t;		/* layer thickness [mm] */
	double E1, E2, G12, v12;	/* mech. properties [MPa] */
	double cte_x, cte_y, cte_xy;	/* thermal properties [1/K] */
	double m, n;
	double Q_11, Q_12,Q_16;	/* transformed reduced stiffness matrix */
	double Q_22, Q_26, Q_66;
	double rho;		/* lamina density */
	/* derived layer heights, used for calculating ABD matrix. */
	double z0, z2, z3;	/* top, z_k-z_{k-1}, z_k^2-z_{k-1}^2, etc. */
};

/* Properties of a laminate. */
struct laminate 
{
	struct list l;		/* list node, _must_ be the first member */
	char *name;
	struct matrix *matrix;	/* the chosen matrix */
	double vf;		/* fiber volume fraction */
	double wf;		/* fiber weight fraction */
	double rc;		/* resin consumption [gr/m2] */
	int nl;			/* number of lamina */
	double t;		/* laminate thickness */
	double rho;		/* laminate density */
	double ABD[6][6];	/* laminate stiffness matrix */
	double abd[6][6];	/* laminate compliance matrix */
	double Ex, Ey, Gxy, Vxy, Vyx;	/* engineering properties */
	double Nt_x, Nt_y, Nt_xy;	/* unti thermal stress resultants */
	double cte_x, cte_y;	/* thermal properties */
	struct list *lamina;	/* starting at the top */
};

/* global variables */
extern struct list  *fiberlist;		/* list of fibers */
extern struct list  *matrixlist;	/* list of matrix materials */
extern struct list  *laminatelist;	/* list of laminates */
/*extern char *outfname;*//* name of output file, or NULL for stdout*/

/***** functions *****/
/* utils.c */
/* Report a fatal error on the standard error stream and terminate the
 * program. */
extern void fatal(const char *fmt, ...);
/* Give a warning on the standard error stream. */
extern void warning(const char *fmt, ...);
/* Read a file into memory. */
extern char *ftobuf(const char *name, unsigned *len);
/* Free the memory that contains the file. */
extern void freefbuf(char *buf);
/* Allocate memory, and terminate the program if allocation fails. */
extern void *getmem(unsigned size);
/* Find a fiber by name in the list of fibers. Return NULL if not found */
extern struct fiber *findfiber(const char *name);
/* Find a matrix by name in the list of matrix materials. Return NULL if
 * not found */
extern struct matrix *findmatrix(const char *name);
/* create an output filename from a given input filename. */
/*extern char *makeoutfname(const char *name, const char *ext);*/

/* parse.c */
/* Open a file for parsing. */
extern void parse_open(const char *name);
/* Parse a single line. Returns != 0 at EOF */
extern int parse_line(void);
/* Stop parsing and close the file. */
extern void parse_close(void);

/* calc.c */
/* Calculate the elastic properties of the k-th lamina. */
extern void lamina_properties(struct laminate *pl, int k);
/* Calculate the the geometric properties of the k-th layer, necessary for
 * calculating the ABD matrix. This should be called in sequence 0 - N, and
 * after the calculation of the lamina properties, since the calculaties
 * needs certain values from the previous layer. */
extern void lamina_geometry(struct laminate *pl, int k);
/* Calculates the ABD matrix, mechanical and thermal properties, after the
 * lamina properties and geometries have been determined. */
extern void ABD(struct laminate *pl);

/* text_out.c */
/* Print the results in text format */
extern void text_output(const struct laminate *pl, unsigned flags);
/* latex_out.c */
/* Print the results in LaTeX2e format */
extern void latex_output(const struct laminate *pl, unsigned flags);
/* html_out.c */
/* Print the results in HTML format */
extern void html_output(const struct laminate *pl, unsigned flags);

END_C_DECLS
#endif				/* LAMPROP_H */
/* EOF lamprop.h */
