/* -*- c -*-
 * Time-stamp: <2005-05-15 12:47:14 rsmith>
 * 
 * lamprop/main.c
 * Copyright (C) 2002--2005 R.F. Smith <rsmith@xs4all.nl>. 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* If you don't want debugging, #define NDEBUG */

#include <assert.h>	/* for assert() */
#include <stdio.h>	/* for printf() */
#include <string.h>
#include "macros.h"
#include "lamprop.h"

#ifndef PACKAGE
#error You must define PACKAGE as the name of the application!
#endif

#ifndef VERSION
#error You must define VERSION as the version of the application!
#endif


/* This enables you to see which files have been used in building a
 * binary. N.B, __attribute__ is a GCC extension, so we test for __GNUC__. */
#ifdef __GNUC__
static char *SCMID __attribute__ ((unused)) = 
"$Id: main.c 77 2005-05-15 13:49:30Z rsmith $";
#endif

extern void print_license(FILE * f);

static void usage(void);
static void help(void);
static void readfile(const char *fname);

static char *usage_string = 
"Usage: "PACKAGE" [-l|-h][-e|-m][-?|-L] file(s)\n";

static void (*outputfn)(const struct laminate *, unsigned) = text_output;

int main(int argc, char *argv[])
{
	int n, nopt = 0;
	struct laminate *pl;
	unsigned flags = 0;
	double vol, fw;
	
	debug("parsing options..");
	for (n=1; n<argc; n++) {
		if (strcmp(argv[n], "-l") == 0) {
			nopt++;
			if (outputfn == text_output) {
				outputfn = latex_output;
			}
		} else if (strcmp(argv[n], "-h") == 0) {
			nopt++;
			if (outputfn == text_output) {
				outputfn = html_output;
			}
		} else if (strcmp(argv[n], "-e") == 0) {
			nopt++;
			flags |= OF_ENGPROP;
		} else if (strcmp(argv[n], "-m") == 0) {
			nopt++;
			flags |= OF_MAT;
		} else if (strcmp(argv[n], "-?") == 0) {
			help();
			return 0;
		} else if (strcmp(argv[n], "-L") == 0) {
			print_license(stderr);
			return 0;
		} else {
			break;
		}
	}
	if ((flags & (OF_ENGPROP|OF_MAT)) == 0) {
		flags = OF_ENGPROP|OF_MAT;
	}

	if (argc-nopt <2) {
		usage();
		return 0;
	}

	for (; n<argc; n++) {
		debug("reading file %s", argv[n]);
		readfile(argv[n]);
	}
	pl = (struct laminate*)list_tail(LISTP(laminatelist));
	while (pl != NULL) {
		fw = 0.0;
		for (n=0; n<pl->nl; n++) {
			lamina_properties(pl, n);
		}
		for (n=0; n<pl->nl; n++) {
			lamina_geometry(pl, n);
			fw += ((struct lamina *)list_nth 
			       (LISTP(pl->lamina), n))->weight;
		}
		vol = 10000.0*pl->t/10.0;	/* volume of 1 m2 [cm3] */
		pl->rc = pl->matrix->rho*(1.0-pl->vf)*vol; /* [gr/m2] */
		pl->wf = fw/(fw+pl->rc);
		ABD(pl);
		outputfn(pl, flags);
		pl = (struct laminate*)list_prev(LISTP(pl));
	}
	return 0;
}

void usage(void)
{
	fputs(PACKAGE" "VERSION"\n", stderr);
	fputs(usage_string, stderr);
	fputs("Type \""PACKAGE" -?\" for an overview of the options.\n\n", 
	      stderr);
}

void help(void)
{
	fputs(PACKAGE" "VERSION"\n", stderr);
	fputs(usage_string, stderr);
	fputs("-l: output the results in LaTeX format.\n", stderr);
	fputs("-h: output the results in HTML format.\n", stderr);
	fputs("-e: produce only the layers and "\
		"engineering properties.\n", stderr);
	fputs("-m: produce only the ABD and abd matrices.\n", stderr);
	fputs("-?: print this help message.\n", stderr);
	fputs("-L: print the license terms.\n", stderr);
	fputs("See "PACKAGE"(1) for information about the program,\n", stderr);
	fputs("and "PACKAGE"(5) about the input file format.\n", stderr);
}

void readfile(const char *fname)
{
	int rc;
	assert(fname!=NULL);
	parse_open(fname);
	do {
		rc = parse_line();
	} while (rc == 0);
	parse_close();
}
/* EOF main.c */
