/* -*- c -*-
 * Time-stamp: <2005-04-16 22:14:35 rsmith>
 * 
 * lamprop/parse.c
 * Copyright (C) 2002--2005 R.F. Smith <rsmith@xs4all.nl>. 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <assert.h>
#include <errno.h>
#ifdef __WIN32__
#include <malloc.h>
#endif
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>

#include "macros.h"
#include "lamprop.h"

/* This enables you to see which files have been used in building a
 * binary. N.B, __attribute is a GCC extension, so we test for __GNUC__. */
#ifdef __GNUC__
static char *SCMID __attribute__ ((unused)) = 
"$Id: parse.c 33 2005-04-16 21:47:48Z rsmith $";
#endif

static char *fname = NULL;
static char *buf = NULL, *end;
static char *currline = NULL;
static unsigned len = 0;

static char *skip_ws(const char *pc);
static char *next_line(char *pc);

/* convert n tokens to double, and the rest to a string. Feed it a list of
 * n+1 pointers to store the data. */
static void conv(const char *str, int n, ...);

/* skip all whitespace to end of buffer */
char *skip_ws(const char *pc)
{
	assert(pc != NULL);
	while (isspace((int)*pc) && *pc != 0) {
		pc++;
	}
	
	return (char*)pc;
}

char *next_line(char *pc)
{
	assert(pc != NULL);
	while (*pc != '\n' && *pc != 0)	/* find newline or end of buffer */
		pc++;
	if (*pc != 0)	/* skip newline */
		pc++;
	return pc;
}

void parse_open(const char *name)
{
	unsigned l;

	if (buf) {	/* already busy */
		errno = EDEADLK;
		fatal("Trying to read two files at once");
	}
	fname = (char*)name;
	buf = ftobuf(fname, &l);
	end = buf + l-1;
	currline = buf;	/* at the beginning of a line */
	debug("file %s opened for parsing", name);
}

int parse_line(void)
{
	char token;
	char *bound;
	struct fiber *pf;
	struct matrix *pm;
	struct laminate *plam;
	struct lamina *pla;
	char *name;
	assert(buf != NULL);
	assert(currline != NULL);
	currline = skip_ws(currline);
	if (*currline == (char)0) {
		debug("end of file reached");
		return 1;
	}
	bound = next_line(currline);
	if (currline[1] != ':') {	/* comment line, skip */
		currline = bound;
		debug("comment line skipped");
		return 0;
	}
	token = *currline;
	currline = skip_ws(currline+2);
	switch (token) {
	case 'f':
		pf = getmem(sizeof(struct fiber));
		conv(currline, 7, &pf->E1, &pf->E2, &pf->v12, &pf->G12, 
		     &pf->cte1, &pf->cte2, &pf->rho, &pf->name);
		if (findfiber(pf->name) != NULL) {
			warning("fiber \'%s\' already exists. Ignored\n");
			free(pf);
		} else {
			fiberlist = list_append(fiberlist, LISTP(pf));
			debug("found fiber: %s", pf->name);
		}		
		break;
	case 'r':
		pm = getmem(sizeof(struct matrix));
		conv(currline, 4, &pm->E, &pm->v, &pm->cte, &pm->rho, 
		     &pm->name);
		pm->G = pm->E/(2*(1+pm->v));
		if (findmatrix(pm->name) != NULL) {
			warning("resin \'%s\' already exists. Ignored\n");
			free(pm);
		} else {
			matrixlist = list_append(matrixlist, LISTP(pm));
			debug("found resin: %s", pm->name);
		}
		break;
	case 't':
		debug("starting new laminate");
		plam = (struct laminate*)list_head(laminatelist);
		if (plam != NULL && plam->lamina == NULL) {
			warning("ignoring empty laminate \"%s\"\n", 
				plam->name);
			free(plam->name);
			/* remove from the list */
			laminatelist = list_head(laminatelist);
			laminatelist = laminatelist->next;
			if (laminatelist)
				laminatelist->prev = NULL;
			free(plam);
		}
		plam = getmem(sizeof(struct laminate));
		conv(currline, 0, &plam->name);
		laminatelist = list_prepend(laminatelist, LISTP(plam));
		debug("starting new laminate: %s", plam->name);
		break;
	case 'm':
		plam = (struct laminate*)list_head(LISTP(laminatelist));
		if (plam == NULL) {
			warning("matrix before laminate ignored\n");
			break;
		}
		conv(currline, 1, &plam->vf, &name);
		if (plam->vf < 0) {
			fatal("fiber volume fraction cannot be < 0\n");
		}
		if (plam->vf >= 1) {
			fatal("fiber volume fraction cannot be >= 1\n");
		}
		plam->matrix = findmatrix(name);
		if (plam->matrix == NULL) {
			fatal("unknown matrix \"%s\"\n", name);
		}
		debug("resin %s, vf %g for laminate %s", name, plam->vf, 
		      plam->name);
		free(name);
		break;
	case 'l':
		plam = (struct laminate*)list_head(LISTP(laminatelist));
		if (plam == NULL) {
			warning("lamina before laminate ignored\n");
			break;
		}
		pla = getmem(sizeof(struct lamina));
		conv(currline, 2, &pla->weight, &pla->angle, &name);
		pla->angle = pla->angle*M_PI/180.0; /* deg->rad */
		if (pla->weight < 0) {
			fatal("weight of layer %d of \"%s\" < 0\n",
			      plam->nl+1, plam->name);
		}
		pla->f = findfiber(name);
		if (pla->f == NULL) {
			fatal("unknown fiber \"%s\" for layer %d  of \"%s\"\n",
			      name, plam->nl+1, plam->name);
		}
		free(name);
		if (pla->f != NULL) {
			plam->lamina = list_append(plam->lamina, LISTP(pla));
			plam->nl++;
			debug("adding lamina to \'%s\'", plam->name);
		} else {
			warning("lamina with unknown fiber ignored\n");
			free(pla);
		}
		break;
	case 'b':
		warning("balanced action not implemented yet.\n");
		break;
	case 's':
		warning("symmetric action not implemented yet.\n");
		break;
	default:	/* unknown token */
		warning("unknown token \'%c:\'.\n", token);
		break;
	}
	currline = bound;
	return 0;
}

void parse_close(void)
{
	assert(buf!=NULL);
	freefbuf(buf);
	fname = NULL; buf = NULL; currline = NULL; len = 0;
}

static void conv(const char *str, int n, ...)
{
	va_list ap;
	char *strend, *dupstr, *endptr, **ppname, *name;
	int dlen, t;
	double *pd, cv;
	/* duplicate the string */
	assert(str!=NULL);
	assert(n>=0);
	strend = next_line((char*)str);
	strend--;
	dlen = strend-str+1;
	dupstr = alloca(dlen+1);
	if (dupstr == NULL) {
		fatal("allocating %d bytes on the stack failed.\n", dlen+1);
	}
	strncpy(dupstr, str, dlen);
	dupstr[dlen] = (char)0;
	strend = &dupstr[dlen-1];
	/* convert the doubles */
	va_start(ap, n);
	for (t=0; t<n; t++) { /* convert the tokens to doubles */
		pd = va_arg(ap, double*);
		cv = strtod(dupstr, &endptr);
		if (errno == ERANGE || endptr == dupstr) {
			va_end(ap);
			debug("conversion %d failed", t);
			fatal("overflow or faulty conversion @ \"%s\"\n",
			       dupstr);
		}
		*pd = cv;
		dupstr = endptr;
		debug("conversion %d succeeded: %g", t, cv);
	}
	dupstr = skip_ws(dupstr);
	/* convert the rest of the tokens to a string */
	ppname = va_arg(ap,char**);
	while (isspace((int)*strend) && strend > dupstr) {
		strend--;
	}
	if (strend == dupstr) {
		fatal("name is empty.\n");
	}
	dlen = strend-dupstr+1;
	name = getmem(dlen+1);
	strncpy(name, dupstr, dlen);
	*ppname = name;
	debug("found name \'%s\'", name);
	va_end(ap);
	return;
}

/* EOF parse.c */
